import java.util.*;


public class ExampleUtilities
{
	private static final char[] HEX_ARRAY = "0123456789ABCDEF".toCharArray();
	
	public static int MENU_TERMINATED = 99;
	
	public static int MenuSelection(String menuTitle, ArrayList<String> menuOptions, MenuOption outOption)
	{
		int res = 0;
		char selection = '\0';
		boolean doLoop = true;

		outOption.clear();

		while(doLoop)
		{

			ExampleUtilities.clearConsole();
			
			System.out.println("[" + menuTitle + "]\n");
			//display menu options.
			for(int x = 0 ; x < menuOptions.size();x++)
			{
				System.out.println( String.format("%d) %s",x+1,menuOptions.get(x)));
			}
			System.out.println("0) Exit\n");
			System.out.print("Select option: ");
	
			try
			{			
				Scanner scnr= new Scanner(System.in);
				selection = scnr.next().charAt(0);
				if( selection == '0')
				{
					res = MENU_TERMINATED;
					doLoop = false;
					continue;
				}
			
				int option = selection - '0';
				System.out.println(String.format("OPTION %d",option));
				if(option < 1 || option > menuOptions.size())
				{
					System.out.println("Invalid menu selection provided");
					ExampleUtilities.pressEnterToContinue();
					continue;
				}
								
				MenuOption yncOpt = new MenuOption();				
				ExampleUtilities.YNCSelection(String.format( "is the selected option '%d) %s' correct",option,menuOptions.get(option-1)), yncOpt);
				if(yncOpt.yncSelection == 'c' || yncOpt.yncSelection == 'C')
				{
					res = MENU_TERMINATED;
					doLoop = false;
					continue;
				}
				else if(yncOpt.yncSelection == 'y' || yncOpt.yncSelection == 'Y')
				{
					res = 0;
					outOption.selectedIndex = option -1;
					doLoop = false;
					continue;
					
				}
			
			}
			catch (final Exception e)
			{
				//  Handle any exceptions.
			}			
			
			
		}
	
		return res;
	}	
	
	public static String BytesToHex(byte[] bytes)
	{
		char[] hexChars = new char[bytes.length * 2];
		for (int j = 0; j < bytes.length; j++) {
			int v = bytes[j] & 0xFF;
			hexChars[j * 2] = HEX_ARRAY[v >>> 4];
			hexChars[j * 2 + 1] = HEX_ARRAY[v & 0x0F];
		}
		return new String(hexChars);
	}
	
	public static String ByteToHex(byte bval)
	{
		char []hexChars = new char[2];
		int v = bval & 0xFF;
		hexChars[0] = HEX_ARRAY[v >>> 4];
		hexChars[1] = HEX_ARRAY[v & 0x0F];
		
		return new String(hexChars);
	}
	
	
	public static String HexDumpWithASCIItoString(byte[] buf)
	{

		long val = 0;
		long bufSz = 0;
		StringBuffer ss = new StringBuffer();
		StringBuffer ss2 = new StringBuffer();
		int x = 0;
		
		if((buf == null) || buf.length == 0 )
		{
			return null;
		}
		else
		{
			bufSz = buf.length;
			val = bufSz % 8;
		
			if(val > 0)
			{
				val = (((8 - (val)) * 5) + 4) - 1;
			}
			else
				val = 3;
			
			for(x = 0 ; x < bufSz; x++)
			{
				if(x > 0)
				{
					if(x % 8 == 0)
					{
						ss2.append("\"");
						ss.append("   " + ss2.toString() + "\n" + padLeftZeros(Integer.toString(x),8) + "    ");
						ss2.setLength(0);
						ss2 .append("\"");
					}
				}
				else
				{
					ss.append("00000000    ");
					ss2.append("\"");
				}
				
				ss.append("0x" + padLeftZeros(ByteToHex(buf[x]),2) + " ");
				
				if(buf[x] >=32 && buf[x] <= 126)
					ss2.append((char)buf[x]);
				else
					ss2.append(".");
			}
			
			if(ss2.length() < 10)
			{
				for(x = ss2.length(); x < 10 - 1;x++)
				{
					ss2.append(" ");
				}
				ss2.append("\"");
			}
		
			for(x = 0 ; x < val ;x++)
				ss.append(" ");
			ss.append(ss2);
			
		}
		
		return ss.toString();
	}
	
	public static String padLeftZeros(String inputString, int length) 
	{
		if (inputString.length() >= length)
		{
			return inputString;
		}
		StringBuilder sb = new StringBuilder();
		while (sb.length() < length - inputString.length())
		{
			sb.append('0');
		}
		
		sb.append(inputString);

		return sb.toString();
	}	


	public static int GetStringValue(String prefix, String question , MenuOption outOption)
	{
		int res = 0;
		String tmpStr = "";
		boolean doLoop = true;

		outOption.clear();


		while(doLoop)
		{

			ExampleUtilities.clearConsole();
			
			System.out.print(prefix + ": ");
				Scanner scnr= new Scanner(System.in);
				tmpStr = scnr.next();
	
			MenuOption yncOpt = new MenuOption();
			ExampleUtilities.YNCSelection(String.format(question,tmpStr),yncOpt);
			if(yncOpt.yncSelection == 'c' || yncOpt.yncSelection == 'C')
			{
				res = MENU_TERMINATED;
				doLoop = false;
				continue;
			}
			else if(yncOpt.yncSelection == 'y' || yncOpt.yncSelection == 'Y')
			{
				res = 0;
				outOption. inputString = tmpStr;
				doLoop = false;
				continue;
				
			}
			
			
		}
	
		return res;		
		
	}

	public static int YNCSelection(String question, MenuOption outOption)
	{
		char selection = '\0';
		boolean doLoop = true;

		outOption.clear();


		while(doLoop)
		{

			ExampleUtilities.clearConsole();
			
			System.out.print(question + "? (y/n/c): ");
	
			try
			{			
				Scanner scnr= new Scanner(System.in);
				selection = scnr.next().charAt(0);
			
				if(	selection !='y' && selection !='Y' &&
					selection !='n' && selection !='N' &&
					selection !='c' && selection !='C')
				{
					System.out.println("Invalid menu selection provided");
					ExampleUtilities.pressEnterToContinue();
					continue;
				}
				
				outOption.yncSelection = selection;
				doLoop = false;
				continue;
				
			
			}
			catch (final Exception e)
			{
				//  Handle any exceptions.
			}			
			
			
		}
	
		return 0;
	}	



	public final static void clearConsole()
	{
		try
		{
			final String os = System.getProperty("os.name");

			if (os.contains("Windows"))
			{
				System.out.println(new String(new char[50]).replace("\0", "\r\n")); 
			}
			else
			{
				Runtime.getRuntime().exec("clear");
			}
		}
		catch (final Exception e)
		{
			//  Handle any exceptions.
		}
	}
	
	public static void pressEnterToContinue()
	{ 
		System.out.println("Press Enter key to continue...");
		try
		{
			System.in.read();
		}  
		catch(Exception e)
		{}  
	}	

		
	
}