
import uk.gov.nhse.vascard.*;
import java.util.*;


public class SimpleSigningStringResultExamples
{

	//
	// Basic examples to perform single Digital Signing Operations using 
	// the NHS England Vendor Agnostic Signing API
	//
	// Caution: This application has a default pin number for the Smart Card
	// hard coded. You will need to change this is you are to use this example
	// for testing and discovery purposes.
	//
	public static void main(String[] args)
	{

		int res = 0;
		HSVASCardSign objsign = null;
		String readerName;
		String pinNumber;
		String results = "";
		try
		{
			objsign = new HSVASCardSign();
			ArrayList<String> readers = new ArrayList<String>();

			//Preamble require near the start of the application. This must be done 
			//prior to any smart card operations being performed with the application.
			res = objsign.InitSigningEnvironment(SCardInitOptions.ENABLE_PIV3S);
			if(res != 0 )
			{
			throw new Exception("InitSigningEnvironment failed. Error Code: " + String.format("0x%08X", res));
			}

			//Retreives the list of available smart card readers on the platform.
			//This example looks for card readers that have smart cards inserted.
			res = objsign.GetListOfAvailableSmartCardReaders(true,readers);
			if(res != 0)
			{
				throw new Exception("GetListOfAvailableSmartCardReaders failed. Error Code: " + String.format("0x%08X", res));
			}
			
			MenuOption retOpt = new MenuOption();
			res = ExampleUtilities.MenuSelection("Available Smart Card Readers", readers, retOpt);
			if( res == ExampleUtilities.MENU_TERMINATED )
			{
				throw new Exception("User terminated application");				
			}

			//set the selected reader from the menu results.
			readerName = readers.get(retOpt.selectedIndex);

			res = ExampleUtilities.GetStringValue(String.format("Please supply pin number for smart card in reader %s",readerName),"Is the pin number value '%s' correct" , retOpt);
			if( res == ExampleUtilities.MENU_TERMINATED )
			{
				throw new Exception("User terminated application");				
			}

			pinNumber = retOpt.inputString;

			System.out.println("[Card Reader Name] : " + readerName);
			System.out.println("[Pin Number] : " + pinNumber);

				
			//Get the Smart Card Name as reported to the Operating System.
			//Information purposes only in this example.
			StringBuffer cardNameBuffer = new StringBuffer();
			objsign.GetSmartCardName(readerName,cardNameBuffer);
			System.out.println("[Card Name] : " + cardNameBuffer.toString());
			
			//Get the Cryptographic Service provider the Operating System uses
			//in conjuction with this card. Information purposes only in this example.
			StringBuffer cardCSPNameBuffer = new StringBuffer();
			objsign.GetSmartCardCSPName(readerName,cardCSPNameBuffer);
			System.out.println("[Card CSP Name] : " + cardCSPNameBuffer.toString());
				

			//Attempt to validate the Smart Card Pin before use. 
			System.out.println("[Validate Smart Card Pin]");
			res = objsign.IsSmartCardPinValid(readerName, pinNumber);
			if(res != SCardPinState.ValidPin )
			{
				throw new Exception("IsSmartCardPinValid failed");
			}
			
			System.out.println("*** Smart Card Pin Validated ***");
			
			System.out.println("[Check SHA256 Support]");
			AlgorithmSupportResult algoSup = new AlgorithmSupportResult();
			res = objsign.SmartCardSupportsSHA256(readerName, algoSup);
			if(res != 0 )
			{
				System.out.println("SmartCardSupportsSHA256 return value: " + String.format("0x%08X", res)); 
			}
			
			System.out.println("SHA-256 supported: " + algoSup.supported);			
			
			res = SigningExampleDefinitions.SmartCardSigningOutputHexString(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA1);
			if( res != 0)
			{
				results = "SmartCardSigningOutputHexString(SHA1) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results = "SmartCardSigningOutputHexString(SHA1) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputHexString(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA_256);
			if( res != 0)
			{
				results += "SmartCardSigningOutputHexString(SHA256) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputHexString(SHA256) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64String(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA1);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64String(SHA1) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64String(SHA1) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64String(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA_256);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64String(SHA256) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64String(SHA256) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64LineBreakString(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA1);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64LineBreakString(SHA1) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64LineBreakString(SHA1) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64LineBreakString(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA_256);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64LineBreakString(SHA256) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64LineBreakString(SHA256) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64String(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA1);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64String(SHA1) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64String(SHA1) Success!\n"; 
			}

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64String(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA_256);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64String(SHA256) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64String(SHA256) Success!\n"; 
			}			



			res = SigningExampleDefinitions.SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA1);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(SHA1) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(SHA1) Success!\n"; 
			}	

			res = SigningExampleDefinitions.SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(objsign,readerName,pinNumber,HashAlgorithm.CALG_SHA_256);
			if( res != 0)
			{
				results += "SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(SHA256) failed. Return value: " + String.format("0x%08X", res) + "\n"; 
			}
			else
			{
				results += "SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(SHA256) Success!\n"; 
			}			

		}
		catch(Exception ex)
		{
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{		
			//This should be done at the end of your application
			objsign.UninitSigningEnvironment();
		}	

		System.out.println("\n\n[Test Results]");
		System.out.println(results);		

	}

}